@@ -135,17 +135,18 @@ get_identity_sync (GoaOAuth2Provider  *oauth2_provider,
 {
   JsonParser *parser = NULL;
   JsonObject *json_object = NULL;
+  JsonNode *json_member = NULL;
   RestProxy *proxy = NULL;
   RestProxyCall *call = NULL;
   GError *identity_error = NULL;
   gchar *authorization = NULL;
-  gchar *presentation_identity = NULL;
-  gchar *id = NULL;
+  const char *id = NULL;
+  const char *presentation_identity = NULL;
   gchar *ret = NULL;
 
   authorization = g_strconcat ("Bearer ", access_token, NULL);
 
-  proxy = goa_rest_proxy_new ("https://graph.microsoft.com/v1.0/me/drive", FALSE);
+  proxy = goa_rest_proxy_new ("https://graph.microsoft.com/v1.0/me", FALSE);
   call = rest_proxy_new_call (proxy);
   rest_proxy_call_set_method (call, "GET");
   rest_proxy_call_add_header (call, "Authorization", authorization);
@@ -182,46 +183,60 @@ get_identity_sync (GoaOAuth2Provider  *oauth2_provider,
       goto out;
     }
 
-  json_object = json_node_get_object (json_parser_get_root (parser));
-  if (!json_object_has_member (json_object, "owner"))
+  json_member = json_parser_get_root (parser);
+  if (json_member == NULL || !JSON_NODE_HOLDS_OBJECT (json_member))
     {
-      g_debug ("Did not find owner in JSON data");
+      g_debug ("%s(): expected root node to be an object", G_STRFUNC);
       g_set_error (error,
                    GOA_ERROR,
                    GOA_ERROR_FAILED,
                    _("Could not parse response"));
       goto out;
     }
+  json_object = json_node_get_object (json_member);
 
-  json_object = json_object_get_object_member (json_object, "owner");
-  if (!json_object_has_member (json_object, "user"))
+  // Use the "id" field as the "Identity"
+  json_member = json_object_get_member (json_object, "id");
+  if (json_member == NULL || json_node_get_value_type (json_member) != G_TYPE_STRING)
     {
-      g_debug ("Did not find user in JSON data");
+      g_debug ("%s(): expected \"id\" field holding a string", G_STRFUNC);
       g_set_error (error,
                    GOA_ERROR,
                    GOA_ERROR_FAILED,
                    _("Could not parse response"));
       goto out;
     }
-  json_object = json_object_get_object_member (json_object, "user");
+  id = json_node_get_string (json_member);
 
-  id = g_strdup (json_object_get_string_member (json_object, "id"));
-
-  if (json_object_has_member (json_object, "email"))
+  // Prefer "mail" then "displayName" for "PresentationIdentity", failing if neither is available
+  json_member = json_object_get_member (json_object, "mail");
+  if (json_member != NULL && json_node_get_value_type (json_member) == G_TYPE_STRING)
     {
-      presentation_identity = g_strdup (json_object_get_string_member (json_object, "email"));
-    }
-  else
-    {
-      presentation_identity = g_strdup (json_object_get_string_member (json_object, "displayName"));
+      presentation_identity = json_node_get_string (json_member);
     }
 
-  ret = id;
-  id = NULL;
+  if (presentation_identity == NULL)
+    {
+      json_member = json_object_get_member (json_object, "displayName");
+      if (json_member != NULL && json_node_get_value_type (json_member) == G_TYPE_STRING)
+        {
+          presentation_identity = json_node_get_string (json_member);
+        }
+      else
+        {
+          g_debug ("%s(): expected \"mail\" or \"displayName\" field holding a string", G_STRFUNC);
+          g_set_error (error,
+                       GOA_ERROR,
+                       GOA_ERROR_FAILED,
+                       _("Could not parse response"));
+          goto out;
+        }
+    }
+
+  ret = g_strdup (id);
   if (out_presentation_identity != NULL)
     {
-      *out_presentation_identity = presentation_identity;
-      presentation_identity = NULL;
+      *out_presentation_identity = g_strdup (presentation_identity);
     }
 
 out:
@@ -230,8 +245,6 @@ out:
   g_clear_object (&call);
   g_clear_object (&proxy);
   g_free (authorization);
-  g_free (id);
-  g_free (presentation_identity);
   return ret;
 }
 
@@ -292,19 +305,20 @@ out:
 }
 
 static void
-add_account_key_values (GoaOAuth2Provider *oauth2_provider,
+add_account_key_values (GoaOAuth2Provider *provider,
                         GVariantBuilder   *builder)
 {
-  GoaMsGraphProvider *self = GOA_MS_GRAPH_PROVIDER (oauth2_provider);
+  g_assert (GOA_IS_MS_GRAPH_PROVIDER (provider));
+  g_assert (builder != NULL);
 
   g_variant_builder_add (builder, "{ss}", "FilesEnabled", "true");
 
-  g_variant_builder_add (builder, "{ss}", "OAuth2AuthorizationUri", self->authorization_uri);
-  g_variant_builder_add (builder, "{ss}", "OAuth2TokenUri", self->token_uri);
-  g_variant_builder_add (builder, "{ss}", "OAuth2ClientId", self->client_id);
-  g_variant_builder_add (builder, "{ss}", "OAuth2RedirectUri", self->redirect_uri);
-  g_variant_builder_add (builder, "{ss}", "OAuth2ClientSecret", "");
-
+  /* The class getter implementations are used, since they handle fallbacks */
+  g_variant_builder_add (builder, "{ss}", "OAuth2AuthorizationUri", goa_oauth2_provider_get_authorization_uri (provider));
+  g_variant_builder_add (builder, "{ss}", "OAuth2TokenUri", goa_oauth2_provider_get_token_uri (provider));
+  g_variant_builder_add (builder, "{ss}", "OAuth2ClientId", goa_oauth2_provider_get_client_id (provider));
+  g_variant_builder_add (builder, "{ss}", "OAuth2RedirectUri", goa_oauth2_provider_get_redirect_uri (provider));
+  g_variant_builder_add (builder, "{ss}", "OAuth2ClientSecret", "" /* always empty */);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -315,9 +329,8 @@ typedef struct
   GoaClient *client;
   GoaObject *object;
 
-  GtkWidget *issuer_combobox;
   GtkWidget *client_id_entry;
-  GtkWidget *custom_issuer_entry;
+  GtkWidget *tenant_id_entry;
 } AccountData;
 
 static void
@@ -358,28 +371,18 @@ create_account_details_ui (GoaProvider *provider,
       GtkWidget *group;
       GtkWidget *button;
 
-      /* NOTE: In case further types needs to be added, ensure that the
-       * code in add_account_action_cb is also adjusted.
-       */
-      static const char * const types[] = {\
-        /* Translators: Microsoft account issuer type */
-        N_("Common"),
-        /* Translators: Microsoft account issuer type */
-        N_("Custom"),
-        NULL
-      };
-
       goa_provider_dialog_add_page (dialog,
                                     NULL, // provider name
-                                    _("Connect to a Microsoft 365 provider to access calendars, contacts and files"));
+                                    _("Connect to a Microsoft 365 provider to access files"));
+
+      group = goa_provider_dialog_add_group (dialog, _("Authorization Details"));
+      /* Translators: See https://learn.microsoft.com/globalization/reference/microsoft-terminology */
+      adw_preferences_group_set_description (ADW_PREFERENCES_GROUP (group), _("A custom Client or Tenant ID may need to be provided depending on the settings for your organization"));
 
-      group = goa_provider_dialog_add_group (dialog, NULL);
       data->client_id_entry = goa_provider_dialog_add_entry (dialog, group, _("_Client ID"));
-
-      group = goa_provider_dialog_add_group (dialog, _("Advanced"));
-      data->issuer_combobox = goa_provider_dialog_add_combo (dialog, group, _("_Issuer"), (GStrv) types);
-      data->custom_issuer_entry = goa_provider_dialog_add_entry (dialog, group, _("C_ustom Issuer"));
-      goa_provider_dialog_add_description (dialog, data->custom_issuer_entry, _("Example provider: example.com"));
+      /* Translators: See https://learn.microsoft.com/globalization/reference/microsoft-terminology */
+      data->tenant_id_entry = goa_provider_dialog_add_entry (dialog, group, _("_Tenant ID"));
+      goa_provider_dialog_add_description (dialog, data->tenant_id_entry, _("Example ID: 00000000-0000-0000-0000-000000000000"));
 
       button = gtk_window_get_default_widget (GTK_WINDOW (dialog));
       gtk_button_set_label (GTK_BUTTON (button), _("_Sign in…"));
@@ -420,7 +423,7 @@ add_account_action_cb (GoaProviderDialog *dialog,
   GCancellable *cancellable = g_task_get_cancellable (task);
   const char *client_id;
   g_autofree char *issuer = NULL;
-  g_autoptr(GError) error = NULL;
+  const char *tenant;
   char *tmp;
 
   if (goa_provider_dialog_get_state (data->dialog) != GOA_DIALOG_BUSY)
@@ -434,14 +437,8 @@ add_account_action_cb (GoaProviderDialog *dialog,
   g_clear_pointer (&self->authorization_uri, g_free);
   g_clear_pointer (&self->token_uri, g_free);
 
-  if (adw_combo_row_get_selected (ADW_COMBO_ROW (data->issuer_combobox)) == 0)
-    {
-      issuer = g_strdup ("common");
-    }
-  else
-    {
-      issuer = g_strdup (gtk_editable_get_text (GTK_EDITABLE (data->custom_issuer_entry)));
-    }
+  tenant = gtk_editable_get_text (GTK_EDITABLE (data->tenant_id_entry));
+  issuer = strlen (tenant) == 0 ? g_strdup ("common") : g_strdup (tenant);
 
   tmp = g_uri_escape_string (issuer, NULL, TRUE);
   self->authorization_uri = g_strdup_printf ("https://login.microsoftonline.com/%s/oauth2/v2.0/authorize", tmp);
@@ -484,7 +481,6 @@ add_account (GoaProvider         *provider,
   g_task_set_check_cancellable (task, FALSE);
   g_task_set_source_tag (task, add_account);
   g_task_set_task_data (task, data, account_data_free);
-  goa_provider_task_bind_window (task, GTK_WINDOW (data->dialog));
 
   create_account_details_ui (provider, data, TRUE);
   g_signal_connect_object (data->dialog,
@@ -492,7 +488,7 @@ add_account (GoaProvider         *provider,
                            G_CALLBACK (add_account_action_cb),
                            task,
                            0 /* G_CONNECT_DEFAULT */);
-  gtk_window_present (GTK_WINDOW (data->dialog));
+  goa_provider_task_run_in_dialog (task, data->dialog);
 
   // We chain-up in add_account_parent_cb() once the user input is confirmed
 }
